from pathlib import Path

RESOURCES_DIR = Path.cwd() / "tests" / "variable_definitions" / "resources"
OPENAPI_DIR = RESOURCES_DIR / "openapi"
VARIABLE_DEFINITION_EDITING_FILES_DIR = (
    RESOURCES_DIR / "variable_definition_editing_files"
)
